import ccxt
import logging
logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s', level=logging.INFO)

class Binance(ccxt.binance):

    def __init__(self, key, secret):
        self.key=key
        self.secret=secret
        super().__init__({'apiKey': key, 'secret': secret,'enableRateLimit': True})
        self.logger=logging.getLogger('Binance')
        self.params={} #'test': True, }
#        self.loadMarkets()

    def Ticker(self,symbol):
        try:
            response = self.fetch_ticker(symbol)
        except Exception as e:
            self.logger.warning('Fetch Ticker for %s failed:%s' %(symbol,str(e)))
            return []
        return response

    def Orders(self,symbol):
        try:
            response = self.fetchOrders(symbol)
        except Exception as e:
            self.logger.warning('Fetch Orders for %s failed:%s' %(symbol,str(e)))
            return []
        return response

    def OpenOrders(self, symbol):
        try:
            response=self.fetchOpenOrders(symbol)
        except Exception as e:
            self.logger.warning('Fetch Open Orders for %s failed:%s' %(symbol,str(e)))
            return []
        return response

    def OpenBuys(self, symbol):
        orders=self.OpenOrders(symbol)
        buys=[]
        for order in orders:
            if order['side']=='buy':
                buys.append(order)
        return buys

    def OpenSells(self, symbol):
        orders=self.OpenOrders(symbol)
        sells=[]
        for order in orders:
            if order['side']=='sell':
                sells.append(order)
        return sells

    def MinCost(self,symbol):
        return self.markets[symbol]['limits']['cost']['min']
#        return 10

    def Buy(self, pair, amount, price):
        free=self.Balance(pair,'quote')
        if amount>free:
            return []
        try:
            response =  self.createLimitBuyOrder(pair, amount, price,self.params)
        except Exception as e:
            self.logger.warning('Create Buy for %s failed:%s' %(symbol,str(e)))
            return []
        return response

    def Sell(self, pair, amount, price):
        free=self.Balance(pair,'base')
        if amount>free:
            return []
        try:
            response =  self.createLimitSellOrder(pair, amount, price, self.params)
        except Exception as e:
            self.logger.warning('Create Sell for %s failed:%s' %(symbol,str(e)))
            return []
        return response

    def Count(self, symbol, type):
        try:
            orders=self.fetchOpenOrders(symbol)
        except Exception as e:
            self.logger.warning('Open Orders %s failed:%s' %(symbol,str(e)))
            return 0

        count=0
        for order in orders:
            if order['side']==type:
                count=count+1
        return count

    def CancelAll(self, symbol):
        for order in self.OpenOrders(symbol):
            self.Cancel(symbol,order['id'])
            
    def Cancel(self, symbol, id):
        try:
            response = self.cancelOrder(id, symbol)
        except Exception as e:
            self.logger.warning('Cancel Position %s %s failed:%s' %(symbol,id,str(e)))
            return []
        return response
        
    def Highest(self,symbol,side='buy'):
        try:
            if side=='buy':
                orders=self.OpenBuys(symbol)
            else:
                orders=self.OpenSells(symbol)
        except Exception as e:
            self.logger.warning('Open Order %s failed: %s' %(symbol, str(e)))
            return []
        
        if len(orders)==0:
            return orders

        highest=orders[0]
        for order in orders:
            if order['side']!=side:
                continue
            if order['price']>highest['price']:
                highest=order
        return highest

    def Lowest(self,symbol,side='buy'):
        try:
            if side=='buy':
                orders=self.OpenBuys(symbol)
            else:
                orders=self.OpenSells(symbol)
        except Exception as e:
            self.logger.warning('Open Orders %s failed: %s' %(symbol, str(e)))
            return []
        
        if len(orders)==0:
            return orders

        lowest=orders[0]
        for order in orders:
            if order['side']!=side:
                continue
            if order['price']<lowest['price']:
                lowest=order
        return lowest

    def Balance(self, symbol, type='quote'): #quote or base
        currency=self.markets[symbol][type]
        try:
            response = self.fetchBalance()[currency]['free']
        except Exception as e:
            self.logger.warning('Fetch Balance %s failed: %s' %(symbol,str(e)))
            return 0
        return response

    def Profitability(self,symbol):
        return 0

    def Info(self):
        return self.has

    def LastBuyFilled(self,symbol):
        orders=self.fetchClosedOrders(symbol)
        i = len(orders)-1
        while (orders[i]['side']!='buy'):
            i=i-1

        return orders[i]
        
    def LastFilled(self,symbol):
        orders=self.fetchClosedOrders(symbol)
        return orders[-1]
